#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-dpm-disk-server-config v. 1.5.0
#
# Post-installation script for configuring the gLite dpm disk server
# Guillermo Diez-Andino Sancho <Guillermo.Sancho@cern.ch>
#
# Version info: $Id: glite-dpm-disk-server-config.py,v 1.6.2.1 2006/02/11 13:37:56 dimeglio Exp $
#
# Usage: python glite-dpm-disk-server-config [-c|-v|-h|--configure|--start|--stop|--status|--help]
#        -c          print configuration
#        -v          print version
#        --configure configure the services
#        --start     start the services
#        --stop      stopt he services
#        --status    check the services status
#        -h,--help   print usage info
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

# import system libraries
import os,string
import sys
import getopt
import time

from  gridSecurity import gridSecurity
from gLiteInstallerLib import ConfigParams
from gLiteInstallerLib import gLib
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params         			# all config values from the XML file
global glib

class glite_dpm_disk_server:

	def __init__(self):
		self.verbose = 0
		self.version = "1.5.0"
		self.name = "glite-dpm-disk-server"
		self.friendly_name = "gLite Dpm Disk Server"

	#-------------------------------------------------------------------------------
	# Banner 
	#-------------------------------------------------------------------------------
	def banner(self):

		print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
		print "          Configuring the %s" % self.friendly_name
		print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
	#-------------------------------------------------------------------------------
	# Copyright 
	#-------------------------------------------------------------------------------
	def copyright(self):

		print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
		print 'See http://eu-egee.org/partners/ for details on the copyright holders'
		print 'For license conditions see the license file or http://eu-egee.org/license.html'

	#-------------------------------------------------------------------------------
	# Version
	#-------------------------------------------------------------------------------
	def showVersion(self):

		print '\n%s-config  v. %s\n' % (self.name,self.version)
    
	#-------------------------------------------------------------------------------
	# Usage
	#-------------------------------------------------------------------------------
	def usage(self,msg = ""):

		if msg:
			print "\n%s" % (msg)
        
		self.copyright()
		self.showVersion()
    
		print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
		os.environ['GLITE_LOCATION'], self.name, self.name)

		print '    -c, --checkconf     print the service configuration'
		print '    -v, --version       print the version of the configuration script'
		print '    -h, --help          print this usage information'
		print '    --configure         configures the dpm services'
		print '    --start			   start the services'
		print '    --stop			   stop the services'
		print '    --status            shows the status of the services'
		print '\n'
	
	def backup(self,path):
		pathBak = "%s.1" %path
		if os.path.exists(pathBak):
			os.remove(pathBak)
		if os.path.exists(path):
			os.rename(path,pathBak)
			
	def ldConfig(self):
		found = 0
		file = open("/etc/ld.so.conf",'r')
		
		for line in file.xreadlines():
			if not line.strip().startswith('#') and line.strip().find('/opt/globus/lib') != -1:
				found = 1
		file.close()		
		if found == 0:
			file = open("/etc/ld.so.conf",'a')
			file.write("/opt/globus/lib")
			file.close()
			print "Added /opt/globus/lib to the /etc/ld.so.conf file",
			glib.printOkMessage()
			os.system("ldconfig")
	
	def start(self):
		if params['dpm-gsiftp-server.enabled'] == "true":
			service.startService("ftpd:","service dpm-gsiftp start")
		
		if params['rfiod-server.enabled'] == "true":
			service.startService("/opt/lcg/bin/rfiod","service rfiod start")
			
		#-------------------------------------------------------------------
		# Start Servicetool
		#-------------------------------------------------------------------
		if params['rgma.servicetool.activate'] == 'true':                                                                                          
			errorcode = rgmaServicetool.start()

			if (errorcode == 0):
				print "The gLite R-GMA Service Publisher has been started           ",                                                                                          
				glib.printOkMessage()
			else:
				glib.printErrorMessage("Could not start the gLite R-GMA Service Publisher")
				glib.printErrorMessage("Please verify and re-run the script")
				return 1
		return 0

	def stop(self):
		if params['dpm-gsiftp-server.enabled'] == "true":
			service.stopService("ftpd:")
			
		if params['rfiod-server.enabled'] == "true":
			service.stopService("/opt/lcg/bin/rfiod")
			
		#############################################
		#Stop the servicetool
		#############################################
		if params['rgma.servicetool.activate'] == 'true':
			errorcode = rgmaServicetool.stop()
			if (errorcode):
				print "Stopping R-GMA Service Publisher                             ",
				glib.printFailedMessage()
			else:
				print "Stopping R-GMA Service Publisher                             ",
				glib.printOkMessage()
		return 0
		
	def status(self):
		error_level = 0
		if params['dpm-gsiftp-server.enabled'] == "true":
			if service.statusService("ftpd:") != 0:
				error_level = 1
				
		if params['rfiod-server.enabled'] == "true":
			if service.statusService("/opt/lcg/bin/rfiod") != 0:
				error_level = 1

		return error_level
	
	#############################################
	# All the configuration code goes here
	#############################################
	def configure(self):
	
		#############################################
		#Stopping the services
		#############################################
		print "Stopping the services " 
		if self.stop() != 0:
			return 1
		
		#############################################
		#Configuring the security utililities
		#############################################
		if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
			print "\nInstalling gLite Security Utilities               ",
			glib.printFailedMessage()
			return 1
		print "\nInstalling gLite Security Utilities                   ",
		glib.printOkMessage()	
		
		#############################################
		#Checking the host certificates
		#############################################
		if glib.check_certs(params) != 0:
			print "Problems checking the host certificates %s" % service.friendly_name
			sys.exit(1)
		
		#############################################
		#Adding the dpm group
		#############################################
		if self.verbose:
			print "Verifying existence of group %s" % params['dpm.groupname']
		gid = glib.add_group(params['dpm.groupname'],params['dpm.gid'])
		
		if self.verbose:
			print 'Group %s has been added or verified to have GID=%d' % (params['dpm.groupname'], gid)
		
		#############################################
		#Adding the dpm user
		#############################################
		(uid,gid) = glib.add_user(params['dpm.username'],params['dpm.groupname'],params['dpm.uid'])
		if self.verbose:
			print 'User account created                                    ',
			glib.printOkMessage()
		
		#############################################
		#Creating dpm certificates folder
		#############################################
		dpmCertPath = "/etc/grid-security/%s" %params['dpm.username']
		uid,gid = glib.get_user_info(params['dpm.username'])
		if not os.path.exists(dpmCertPath):
			os.mkdir(dpmCertPath,0755)
			os.chown(dpmCertPath,uid,gid)
		else:
			if self.verbose:
				print "The %s folder already exists" %dpmCertPath,
				glib.printOkMessage()
			
		#############################################
		# Copying the certificates to the dpm user folder
		#############################################
		command = "cp %s %s/dpmcert.pem" %(params['host.certificate.file'],dpmCertPath)
		if self.verbose:
			print "Copying host certificates to %s directory" %params['dpm.username']
			print "Executing %s" %command
		os.system(command)
		os.chown("/etc/grid-security/%s/dpmcert.pem" %params['dpm.username'],uid,gid)
		os.chmod("/etc/grid-security/%s/dpmcert.pem" %params['dpm.username'],0644)
		
		command = "cp %s %s/dpmkey.pem" %(params['host.key.file'],dpmCertPath)
		if self.verbose:
			print "Copying host certificate key to %s directory" %params['dpm.username']
			print "Executing %s" %command
		os.system(command) 
		os.chown("/etc/grid-security/%s/dpmkey.pem" %params['dpm.username'],uid,gid)
		os.chmod("/etc/grid-security/%s/dpmkey.pem" %params['dpm.username'],0400)

		if (os.path.exists('/etc/grid-security/%s/dpmcert.pem' %params['dpm.username']) and os.path.exists('/etc/grid-security/%s/dpmkey.pem' %params['dpm.username']))	:
			print "Copied host certificates and key to /etc/grid-security/%s" %params['dpm.username'],
			glib.printOkMessage()
		else:
			print "An error occurred while copying the host certificates to /etc/grid-security/%s" %params['dpm.username'],
			glib.printFailedMessage()
			return 1
		
		#############################################
		#Checking the grid-mapdir
		#############################################
		if self.verbose:
			print "Checking the gridmapdir"
		if not os.path.exists(params['host.gridmap.dir']):
			os.mkdir(params['host.gridmap.dir'])
			if os.path.exists(params['host.gridmap.dir']):
				print "gridmapdir successfully created",
				glib.printOkMessage(),
			else:
				print "problems to create the gridmapdir under %s" %params['host.gridmap.dir'],
				glib.printFailedMessage()
				return 1
		os.system("chown root:%s %s" %(params['dpm.username'],params['host.gridmap.dir']))

		#############################################
		#Checking the grid-mapfile
		#############################################
		if not os.path.exists(params['host.gridmapfile']):
			print("Creating an empty grid-mapfile under %s" %params['host.gridmapfile'])
			os.system("/bin/touch %s" % params['host.gridmapfile'])
		else:
			print("The grid-mapfile exists under %s" %params['host.gridmapfile']),
			glib.printOkMessage()
			
		#############################################
		#Create pool accounts
		#############################################
		ret_val = gsec.create_pool_accounts()
		if ret_val == 1:
			return 1
			
		############################################
		# Configuring the different services (dpm-gsiftp,rfiod)
		############################################
		if params['dpm-gsiftp-server.enabled'] == "true":
			service.configureDPMGSIFTP()

		if params['rfiod-server.enabled'] == "true":
			service.configureRFIOD()
			
		############################################
		# Configuring the rgma servicetool
		############################################
		if self.configureServiceTool() == 1:
			return 1
                                                                                                                                                               
		print "Configuring the R-GMA Service Publisher                      ",
		glib.printOkMessage()

		############################################
		# Creating the shift.conf file
		############################################
		if self.authorization() != 0:
			glib.printErrorMessage("Error when creating the /etc/shift.conf file")
			return 1
		else:
			print "/etc/shift.conf file correctly created",
			glib.printOkMessage()	
		
		return 0
		
	def configureServiceTool(self):
		serviceTypes = ["lcg.dpm.gsiftp-server","lcg.dpm.rfiod-server"]
		serviceScripts = ["/etc/init.d/rfiod status","/etc/init.d/dpm-gsiftp status"]
		
		if params['rgma.servicetool.activate'] == "true":
			for idx in xrange(len(serviceTypes)):
				serviceId = "%s_%s" % (glib.fq_hostname,serviceTypes[idx])
				servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
				glib.printInfoMessage("Configuring servicetool instance %s" %servicetoolInstance)
                                                                                                                                                               
				# set params
				servicetoolInstance.setServiceName(serviceId)
				servicetoolInstance.setServiceType(serviceTypes[idx])
				servicetoolInstance.setServiceVersion(self.version)
				servicetoolInstance.setStatusScript(serviceScripts[idx])
				servicetoolInstance.setEndpoint('not available')

				# add instance to the gLite configuration
				if servicetoolInstance.add():
					return 1
			
			if rgmaServicetool.configure(glib):
				# error in configuring servicetool
				return 1
		return 0	
		
	def authorization(self):
		############################################
		# Creating the shift.conf file
		############################################
		path = "/etc/shift.conf"
		
		#RFIOD authorization
		if (params['rfiod-server.enabled']=="true" and os.environ['HOSTNAME']!=params['dpm-server.host']):
			self.backup(path) 
			file = open(path,"a")
			hostName = params['dpm-server.host'].split(".")[0]
			file.write("RFIOD TRUST %s %s\n" %(hostName,params['dpm-server.host']))
			file.write("RFIOD RTRUST %s %s\n" %(hostName,params['dpm-server.host']))
			file.write("RFIOD WTRUST %s %s\n" %(hostName,params['dpm-server.host']))
			file.write("RFIOD XTRUST %s %s\n" %(hostName,params['dpm-server.host']))
			file.write("RFIOD FTRUST %s %s\n" %(hostName,params['dpm-server.host']))
			file.write("RFIO DAEMONV3_WRMT 1\n")
			file.close()
		return 0			
		
	def startService(self,daemon,startupScript):
		returnCode = 0
		
		self.stopService(daemon)
		os.system(startupScript)
		
		if glib.getPID(daemon) == 0:
			glib.printErrorMessage("The %s daemon has not been properly started" %daemon)
			returnCode = 1
				
		return returnCode	
		
	def stopService(self,daemon):
		returnCode = 0
		
		if glib.getPID(daemon) != 0:
			glib.stopProcess(daemon)
			if glib.getPID(daemon) != 0:
				glib.printErrorMessage("The process %s has not been properly stopped" %daemon)
				returnCode = 1
		else:
			print("The process %s is already stopped" %daemon),
			glib.printOkMessage()
		return returnCode	
	
	def statusService(self,daemon):
		error_level = 0

		pid = glib.getPID(daemon)
		if pid != 0:
			print "The process %s is running" %daemon
		else:
			print "The process %s is not running" %daemon
			error_level = 1
		return error_level	
		
	#############################################
	# Configuring the dpm-gsiftp service
	#############################################
	def configureDPMGSIFTP(self):
		print "Configuring the DPM-GSIFTP server"
		path = "/etc/sysconfig/dpm-gsiftp"
			
		self.backup(path)

		print "Creating the %s configuration file" %path
		file = open(path, 'w')
		file.write("export DPNS_HOST=%s\n" %params['dpns-server.host'])
		file.write("export DPM_HOST=%s\n" %params['dpm-server.host'])
		file.close()
		print("%s configuration file created " % path),
		glib.printOkMessage()
		return 0
	
	#############################################
	# Configuring the RFIOD service
	#############################################	
	def configureRFIOD(self):
		print "Configuring the RFIOD server"
		path = "/etc/sysconfig/rfiod"
		
		self.backup(path)
			
		print "Creating the %s configuration file" %path
		file = open(path, 'w')
		file.write("RUN_RFIOD=\"yes\"\n")
		file.write("export DPNS_HOST=%s\n" %params['dpns-server.host'])
		file.write("export DPM_HOST=%s\n" %params['dpm-server.host'])
		file.write("RFIOLOGFILE=\"%s\"\n" % params['rfiod-server.logfile'])
		file.write("export GRIDMAPDIR=\"%s\"\n" % params['host.gridmap.dir'])
		file.close()
		
		#Adding /opt/globus/lib to /etc/ld.so.conf
		print "Checking the ld.so.conf file"
		self.ldConfig()
		
		print("%s configuration file created " % path),
		glib.printOkMessage()
		return 0
	
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------
def set_env():
	
	glib.export('GLITE_LOCATION');
	glib.export('GLITE_LOCATION_VAR');
	if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
		os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
	glib.export('GLITE_LOCATION_LOG');
	if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
		os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
	glib.export('GLITE_LOCATION_TMP');
	
	if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
		os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

	glib.export('GLOBUS_LOCATION')
		
	# bin and lib paths
	glib.addEnvPath("PATH","%s/bin:%s/sbin,%s/libexec" % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION']))
	glib.addEnvPath("LD_LIBRARY_PATH","%s/lib:%s/lib:/opt/gpt/lib" % (os.environ['GLOBUS_LOCATION'],os.environ['GLITE_LOCATION']))
	
	# Set environment
	glib.setUserEnv()	
	

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':
	# The script must be run as root
	if not os.geteuid()==0:
		glib.printFailedMessage("This script must be run as root")
		sys.exit(1)
	
	# Get an instance of the library class
	glib = gLib()
	
	# Load parameters
	params = ConfigParams() 
	glib.loadConfiguration(["%s/../glite-dpm-disk-server.cfg.xml" % glib.getScriptPath(), "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()],params)
	
	try:
		opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
		for o, a in opts:
			if o == "--siteconfig":
				params['site.config.url'] = a
				break
	except getopt.GetoptError:
		pass
    
	# set verbosity
	if params.has_key('glite.installer.verbose'):
		if (params['glite.installer.verbose'] == "true"):
			verbose=1
		else:
			verbose=0
	else:
		verbose = 0
	glib.verbose = verbose
	
	# Set up the environment
	set_env()
	service = glite_dpm_disk_server()
	
	#Instantiate grdSecurity classes
	gsec = gridSecurity(glib)
	gsec.setVerbosity(verbose)
	gsec.params = params
	
	# Instantiate the rgma servicetool class
	if not params.has_key('rgma.servicetool.activate'):
		params['rgma.servicetool.activate'] = 'true'
	if params['rgma.servicetool.activate'] == 'true':
		rgmaServicetool = gliteRgmaServicetool()
		rgmaServicetool.verbose = verbose
                                                                                                  
	path = '%s/etc' % os.environ['GLITE_LOCATION']
	if not os.path.exists(path):
		glib.printErrorMessage("%s/etc does not exist! Please verify your gLite installation") % os.environ['GLITE_LOCATION']
		sys.exit (1)
	else:
		print '%s/etc exists' % os.environ['GLITE_LOCATION'],
		glib.printOkMessage()

	#Checking that at least one service is enabled
	if params['dpm-gsiftp-server.enabled'] != "true" and params['rfiod-server.enabled'] != "true":
		glib.printErrorMessage("At least one of either the dpm-gsiftp-server.enabled or rfiod-server.enabled should be enabled in the configuration file ")
		sys.exit(1)
	
	# Command line opts if any
	try:
		opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','stop','start','status','configure','siteconfig='])
	except getopt.GetoptError:
		service.usage(msg = "[FAILED] Unknown options(s)")
		sys.exit(1)

	# Check cli options
	for o, a in opts:
		if o in ("-h", "--help"):
			service.usage()
			sys.exit(0)
		if o in ("-v", "--version"):
			service.showVersion()
			sys.exit(0)
		if o in ("-c", "--checkconf"):
			service.copyright()
			service.showVersion()
			glib.print_params(params)
			sys.exit(0)
		if o == "--start":
			if service.start() == 0:
				print "The %s was successfully started" % service.friendly_name,
				glib.printOkMessage()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while starting the %s service" % service.friendly_name)
				sys.exit(1)
		if o == "--stop":
			if service.stop() == 0:
				print("The service %s was successfully stopped" % service.friendly_name),
				glib.printOkMessage()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while stopping the %s service" % service.friendly_name)
				sys.exit(1)
		if o == "--status":
			sys.exit(service.status())
		if o == "--configure":
			if service.configure() == 0:
				print("%s configuration successfully completed " % service.friendly_name),
				glib.printOkMessage()
				glib.registerService()
				sys.exit(0)
			else:
				glib.printErrorMessage("An error occurred while configuring the %s service" % service.friendly_name)
				sys.exit(1)
 
	# Print configuration parameters
	if verbose:
		glib.print_params(params)
		service.copyright()
		service.showVersion()
		service.banner()
	service.usage()	
